@@ -75,6 +75,10 @@ def topology(request):
     # Used to retrieve configuration information (dbdir, confdir...)
     standalone.open()
 
+    def fin():
+        standalone.delete()
+    request.addfinalizer(fin)
+
     # Here we have standalone instance up and running
     return TopologyStandalone(standalone)
 
@@ -387,7 +391,7 @@ def test_dynamic_plugins(topology):
     # Loop on the consumer - waiting for it to catch up
     count = 0
     insync = False
-    while count < 10:
+    while count < 60:
         try:
             # Grab master's max CSN
             entry = replica_inst.search_s(DEFAULT_SUFFIX, ldap.SCOPE_SUBTREE, RUV_FILTER)
@@ -469,25 +473,8 @@ def test_dynamic_plugins(topology):
     log.info('#####################################################\n')
 
 
-def test_dynamic_plugins_final(topology):
-    topology.standalone.delete()
-
-
-def run_isolated():
-    '''
-        run_isolated is used to run these test cases independently of a test scheduler (xunit, py.test..)
-        To run isolated without py.test, you need to
-            - edit this file and comment '@pytest.fixture' line before 'topology' function.
-            - set the installation prefix
-            - run this program
-    '''
-    global installation_prefix
-    installation_prefix = None
-
-    topo = topology(True)
-    test_dynamic_plugins(topo)
-    test_dynamic_plugins_final(topo)
-
-
 if __name__ == '__main__':
-    run_isolated()
+    # Run isolated
+    # -s for DEBUG mode
+    CURRENT_FILE = os.path.realpath(__file__)
+    pytest.main("-s %s" % CURRENT_FILE)
