@@ -1234,9 +1234,9 @@ conn_connect(Repl_Connection *conn)
 	 * initialisation should be done before ever trying to open any connection at all.
 	 */
 	if (conn->transport_flags == TRANSPORT_FLAG_TLS) {
-		secure = 2;
+		secure = SLAPI_LDAP_INIT_FLAG_startTLS;
 	} else if (conn->transport_flags == TRANSPORT_FLAG_SSL) {
-		secure = 1;
+		secure = SLAPI_LDAP_INIT_FLAG_SSL;
 	}
 
 	if (secure > 0) {
@@ -1261,7 +1261,7 @@ conn_connect(Repl_Connection *conn)
 			"%s: Trying %s%s slapi_ldap_init_ext\n",
 			agmt_get_long_name(conn->agmt),
 			secure ? "secure" : "non-secure",
-			(secure == 2) ? " startTLS" : "");
+			(secure == SLAPI_LDAP_INIT_FLAG_startTLS) ? " startTLS" : "");
 		/* shared = 1 because we will read results from a second thread */
 		if (conn->ld) {
 			/* Since we call slapi_ldap_init, we must call slapi_ldap_unbind */
@@ -1279,7 +1279,7 @@ conn_connect(Repl_Connection *conn)
 				"%s: Failed to establish %s%sconnection to the consumer\n",
 				agmt_get_long_name(conn->agmt),
 				secure ? "secure " : "",
-				(secure == 2) ? "startTLS " : "");
+				(secure == SLAPI_LDAP_INIT_FLAG_startTLS) ? "startTLS " : "");
 			goto done;
 		}
 		
@@ -2072,11 +2072,19 @@ bind_and_check_pwp(Repl_Connection *conn, char * binddn, char *password)
 	{
 		if (conn->last_ldap_error != rc)
 		{
+			int log_level = SLAPI_LOG_FATAL;
+			if (conn->last_ldap_error == LDAP_LOCAL_ERROR){
+				/*
+				 * Local errors are not logged by default, so when we recover
+				 * from local errors we must also not log it by default.
+				 */
+				log_level = SLAPI_LOG_REPL;
+			}
 			conn->last_ldap_error = rc;
-			slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name,
-							"%s: Replication bind with %s auth resumed\n",
-							agmt_get_long_name(conn->agmt),
-							mech ? mech : "SIMPLE");
+			slapi_log_error(log_level, repl_plugin_name,
+				"%s: Replication bind with %s auth resumed\n",
+				agmt_get_long_name(conn->agmt),
+				mech ? mech : "SIMPLE");
 		}
 
 		if ( ctrls ) 
@@ -2122,20 +2130,21 @@ bind_and_check_pwp(Repl_Connection *conn, char * binddn, char *password)
 			conn->last_ldap_error = rc;
 			/* errmsg is a pointer directly into the ld structure - do not free */
 			rc = slapi_ldap_get_lderrno( ld, NULL, &errmsg );
-			slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name,
-							"%s: Replication bind with %s auth failed: LDAP error %d (%s) (%s)\n",
-							agmt_get_long_name(conn->agmt),
-							mech ? mech : "SIMPLE", rc,
-							ldap_err2string(rc), errmsg ? errmsg : "");
+			slapi_log_error(rc == LDAP_LOCAL_ERROR ? SLAPI_LOG_REPL : SLAPI_LOG_FATAL,
+				repl_plugin_name,
+				"%s: Replication bind with %s auth failed: LDAP error %d (%s) (%s)\n",
+				agmt_get_long_name(conn->agmt),
+				mech ? mech : "SIMPLE", rc,
+				ldap_err2string(rc), errmsg ? errmsg : "");
 		} else {
 			char *errmsg = NULL;
 			/* errmsg is a pointer directly into the ld structure - do not free */
 			rc = slapi_ldap_get_lderrno( ld, NULL, &errmsg );
 			slapi_log_error(SLAPI_LOG_REPL, repl_plugin_name,
-							"%s: Replication bind with %s auth failed: LDAP error %d (%s) (%s)\n",
-							agmt_get_long_name(conn->agmt),
-							mech ? mech : "SIMPLE", rc,
-							ldap_err2string(rc), errmsg ? errmsg : "");
+				"%s: Replication bind with %s auth failed: LDAP error %d (%s) (%s)\n",
+				agmt_get_long_name(conn->agmt),
+				mech ? mech : "SIMPLE", rc,
+				ldap_err2string(rc), errmsg ? errmsg : "");
 		}
 
 		return (CONN_OPERATION_FAILED);
@@ -2190,7 +2199,6 @@ static void
 repl5_stop_debug_timeout(Slapi_Eq_Context eqctx, int *setlevel)
 {
 	char buf[20];
-	char msg[SLAPI_DSE_RETURNTEXT_SIZE];
 
 	if (eqctx && !*setlevel) {
 		(void)slapi_eq_cancel(eqctx);
@@ -2199,7 +2207,7 @@ repl5_stop_debug_timeout(Slapi_Eq_Context eqctx, int *setlevel)
 	if (s_debug_timeout && s_debug_level && *setlevel) {
 		void config_set_errorlog_level(const char *type, char *buf, char *msg, int apply);
 		sprintf(buf, "%d", 0);
-		config_set_errorlog_level("nsslapd-errorlog-level", buf, msg, 1);
+		config_set_errorlog_level("nsslapd-errorlog-level", buf, NULL, 1);
 	}
 }
 
@@ -2209,11 +2217,10 @@ repl5_debug_timeout_callback(time_t when, void *arg)
 	int *setlevel = (int *)arg;
 	void config_set_errorlog_level(const char *type, char *buf, char *msg, int apply);
 	char buf[20];
-	char msg[SLAPI_DSE_RETURNTEXT_SIZE];
 
 	*setlevel = 1;
 	sprintf(buf, "%d", s_debug_level);
-	config_set_errorlog_level("nsslapd-errorlog-level", buf, msg, 1);
+	config_set_errorlog_level("nsslapd-errorlog-level", buf, NULL, 1);
 
 	slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name, 
 		"repl5_debug_timeout_callback: set debug level to %d at %ld\n",
