@@ -1313,9 +1313,9 @@ windows_conn_connect(Repl_Connection *conn)
 	 * initialisation should be done before ever trying to open any connection at all.
 	 */
 	if (conn->transport_flags == TRANSPORT_FLAG_TLS) {
-        secure = 2;
+        secure = SLAPI_LDAP_INIT_FLAG_startTLS;
     } else if (conn->transport_flags == TRANSPORT_FLAG_SSL) {
-        secure = 1;
+        secure = SLAPI_LDAP_INIT_FLAG_SSL;
     }
  
 	if (secure > 0) {
@@ -1340,7 +1340,7 @@ windows_conn_connect(Repl_Connection *conn)
 			"%s: Trying %s%s slapi_ldap_init_ext\n",
 			agmt_get_long_name(conn->agmt),
 			secure ? "secure" : "non-secure",
-			(secure == 2) ? " startTLS" : "");
+			(secure == SLAPI_LDAP_INIT_FLAG_startTLS) ? " startTLS" : "");
 		
 		conn->ld = slapi_ldap_init_ext(NULL, conn->hostname, conn->port, secure, 0, NULL);
 		if (NULL == conn->ld)
@@ -1353,7 +1353,7 @@ windows_conn_connect(Repl_Connection *conn)
 				"%s: Failed to establish %s%sconnection to the consumer\n",
 				agmt_get_long_name(conn->agmt),
 				secure ? "secure " : "",
-				(secure == 2) ? "startTLS " : "");
+				(secure == SLAPI_LDAP_INIT_FLAG_startTLS) ? "startTLS " : "");
 			goto done;
 		}
 		
@@ -2029,7 +2029,6 @@ static void
 repl5_stop_debug_timeout(Slapi_Eq_Context eqctx, int *setlevel)
 {
 	char buf[20];
-	char msg[SLAPI_DSE_RETURNTEXT_SIZE];
 
 	LDAPDebug( LDAP_DEBUG_TRACE, "=> repl5_stop_debug_timeout\n", 0, 0, 0 );
 
@@ -2040,7 +2039,7 @@ repl5_stop_debug_timeout(Slapi_Eq_Context eqctx, int *setlevel)
 	if (s_debug_timeout && s_debug_level && *setlevel) {
 		/* No longer needed as we are including the one in slap.h */
 		sprintf(buf, "%d", 0);
-		config_set_errorlog_level("nsslapd-errorlog-level", buf, msg, 1);
+		config_set_errorlog_level("nsslapd-errorlog-level", buf, NULL, 1);
 	}
 
 	LDAPDebug( LDAP_DEBUG_TRACE, "<= repl5_stop_debug_timeout\n", 0, 0, 0 );
@@ -2052,13 +2051,12 @@ repl5_debug_timeout_callback(time_t when, void *arg)
 	int *setlevel = (int *)arg;
 	/* No longer needed as we are including the one in slap.h */
 	char buf[20];
-	char msg[SLAPI_DSE_RETURNTEXT_SIZE];
 
 	LDAPDebug( LDAP_DEBUG_TRACE, "=> repl5_debug_timeout_callback\n", 0, 0, 0 );
 
 	*setlevel = 1;
 	sprintf(buf, "%d", s_debug_level);
-	config_set_errorlog_level("nsslapd-errorlog-level", buf, msg, 1);
+	config_set_errorlog_level("nsslapd-errorlog-level", buf, NULL, 1);
 
 	slapi_log_error(SLAPI_LOG_FATAL, windows_repl_plugin_name, 
 		"repl5_debug_timeout_callback: set debug level to %d at %ld\n",
