@@ -66,7 +66,7 @@ mod_op_image (int op)
 #endif
 
 /* an AttrCheckFunc function should return an LDAP result code (LDAP_SUCCESS if all goes well). */
-typedef int (*AttrCheckFunc)(const char *attr_name, char *value, long minval, long maxval, char *errorbuf);
+typedef int (*AttrCheckFunc)(const char *attr_name, char *value, long minval, long maxval, char *errorbuf, size_t ebuflen);
 
 static struct attr_value_check {
 	const char *attr_name; /* the name of the attribute */
@@ -85,7 +85,7 @@ static struct attr_value_check {
 	{CONFIG_PW_WARNING_ATTRIBUTE, check_pw_duration_value, 0, -1},
 	{CONFIG_PW_MINLENGTH_ATTRIBUTE, attr_check_minmax, 2, 512},
 	{CONFIG_PW_MAXFAILURE_ATTRIBUTE, attr_check_minmax, 1, 32767},
-	{CONFIG_PW_INHISTORY_ATTRIBUTE, attr_check_minmax, 2, 24},
+	{CONFIG_PW_INHISTORY_ATTRIBUTE, attr_check_minmax, 1, 24},
 	{CONFIG_PW_LOCKDURATION_ATTRIBUTE, check_pw_duration_value, -1, -1},
 	{CONFIG_PW_RESETFAILURECOUNT_ATTRIBUTE, check_pw_resetfailurecount_value, -1, -1},
 	{CONFIG_PW_GRACELIMIT_ATTRIBUTE, attr_check_minmax, 0, -1},
@@ -219,8 +219,8 @@ do_modify( Slapi_PBlock *pb )
 		mod->mod_op = mod_op;
 		mod->mod_type = slapi_attr_syntax_normalize(type);
 		if ( !mod->mod_type || !*mod->mod_type ) {
-			char ebuf[BUFSIZ];
-			PR_snprintf (ebuf, BUFSIZ, "invalid type '%s'", type);
+			char ebuf[SLAPI_DSE_RETURNTEXT_SIZE];
+			PR_snprintf (ebuf, sizeof(ebuf), "invalid type '%s'", type);
 			op_shared_log_error_access (pb, "MOD", rawdn, ebuf);
 			send_ldap_result( pb, LDAP_INVALID_SYNTAX, NULL, ebuf, 0, NULL );
 			slapi_ch_free((void **)&type);
@@ -496,7 +496,7 @@ slapi_modify_internal_set_pb_ext(Slapi_PBlock *pb, const Slapi_DN *sdn,
 	if (pb == NULL || sdn == NULL || mods == NULL)
 	{
 		slapi_log_error(SLAPI_LOG_FATAL, NULL, 
-						"slapi_modify_internal_set_pb: NULL parameter\n");
+						"slapi_modify_internal_set_pb_ext: NULL parameter\n");
 		return;
 	}
 
@@ -628,7 +628,7 @@ static void op_shared_modify (Slapi_PBlock *pb, int pw_change, char *old_pw)
 	int repl_op, internal_op, lastmod, skip_modified_attrs;
 	char *unhashed_pw_attr = NULL;
 	Slapi_Operation *operation;
-	char errorbuf[BUFSIZ];
+	char errorbuf[SLAPI_DSE_RETURNTEXT_SIZE];
 	int err;
 	LDAPMod *lc_mod = NULL;
 	struct slapdplugin  *p = NULL;
@@ -710,7 +710,8 @@ static void op_shared_modify (Slapi_PBlock *pb, int pw_change, char *old_pw)
 	 * We could be serving multiple database backends.  Select the
 	 * appropriate one.
 	 */
-	if ((err = slapi_mapping_tree_select(pb, &be, &referral, errorbuf)) != LDAP_SUCCESS) {
+	errorbuf[0] = '\0';
+	if ((err = slapi_mapping_tree_select(pb, &be, &referral, errorbuf, sizeof(errorbuf))) != LDAP_SUCCESS) {
 		send_ldap_result(pb, err, NULL, errorbuf, 0, NULL);
 		be = NULL;
 		goto free_and_return;
@@ -765,7 +766,7 @@ static void op_shared_modify (Slapi_PBlock *pb, int pw_change, char *old_pw)
 							 */
 							if ( (err = AttrValueCheckList[i].checkfunc (AttrValueCheckList[i].attr_name,
 								(*tmpmods)->mod_bvalues[0]->bv_val, AttrValueCheckList[i].minval,
-								AttrValueCheckList[i].maxval, errorbuf))
+								AttrValueCheckList[i].maxval, errorbuf, sizeof(errorbuf)))
 								!= LDAP_SUCCESS)
 							{
 								/* return error */
@@ -826,7 +827,7 @@ static void op_shared_modify (Slapi_PBlock *pb, int pw_change, char *old_pw)
 				for ( i = 0; pw_mod->mod_bvalues != NULL && pw_mod->mod_bvalues[i] != NULL; i++ ) {
 					password = slapi_ch_strdup(pw_mod->mod_bvalues[i]->bv_val);
 					pwsp = pw_val2scheme( password, &valpwd, 1 );
-					if(strcmp(pwsp->pws_name, "CLEAR") == 0){
+					if(pwsp == NULL || strcmp(pwsp->pws_name, "CLEAR") == 0){
 						/*
 						 *  CLEAR password
 						 *
@@ -850,7 +851,7 @@ static void op_shared_modify (Slapi_PBlock *pb, int pw_change, char *old_pw)
 								const char *userpwd = slapi_value_get_string(present_values[ii]);
 
 								pass_scheme = pw_val2scheme( (char *)userpwd, &pval, 1 );
-								if(strcmp(pass_scheme->pws_name,"CLEAR")){
+								if(pass_scheme && strcmp(pass_scheme->pws_name,"CLEAR")){
 									/* its encoded, so compare it */
 									if((*(pass_scheme->pws_cmp))( valpwd, pval ) == 0 ){
 									    /*
@@ -911,7 +912,7 @@ static void op_shared_modify (Slapi_PBlock *pb, int pw_change, char *old_pw)
 								 *  provided by the client.
 								 */
 								unhashed_pwsp = pw_val2scheme( (char *)unhashed_pwd, NULL, 1 );
-								if(strcmp(unhashed_pwsp->pws_name, "CLEAR") == 0){
+								if(unhashed_pwsp == NULL || strcmp(unhashed_pwsp->pws_name, "CLEAR") == 0){
 									if((*(pwsp->pws_cmp))((char *)unhashed_pwd , valpwd) == 0 ){
 										/* match, add the delete mod for this particular unhashed userpassword */
 										if (SLAPD_UNHASHED_PW_OFF != config_get_unhashed_pw_switch()) {
@@ -1079,6 +1080,11 @@ static void op_shared_modify (Slapi_PBlock *pb, int pw_change, char *old_pw)
 					operation_out_of_disk_space();
 					goto free_and_return;
 				}
+				/* If the disk is full we don't want to make it worse ... */
+				if (operation_is_flag_set(operation,OP_FLAG_ACTION_LOG_AUDIT))
+				{ 
+					write_auditfail_log_entry(pb); /* Record the operation in the audit log */
+				}
 			}
 		}
 		else
@@ -1150,7 +1156,7 @@ valuearray_init_bervalarray_unhashed_only(struct berval **bvals, Slapi_Value ***
 		*cvals = (Slapi_Value **) slapi_ch_malloc((n + 1) * sizeof(Slapi_Value *));
 		for(i=0,p=0;i<n;i++){
 			pwsp = pw_val2scheme( bvals[i]->bv_val, NULL, 1 );
-			if(strcmp(pwsp->pws_name, "CLEAR") == 0){
+			if(pwsp == NULL || strcmp(pwsp->pws_name, "CLEAR") == 0){
 				(*cvals)[p++] = slapi_value_new_berval(bvals[i]);
 			}
 			free_pw_scheme( pwsp );
@@ -1430,22 +1436,24 @@ hash_rootpw (LDAPMod **mods)
 		if (strcasecmp (mod->mod_type, CONFIG_ROOTPW_ATTRIBUTE) != 0) 
 			continue;
 
-		for (j = 0; mod->mod_bvalues[j] != NULL; j++) {
-			char *val = mod->mod_bvalues[j]->bv_val;
-			char *hashedval = NULL;
-			struct pw_scheme *pws = pw_val2scheme (val, NULL, 0);
-			if (pws) {
-				free_pw_scheme(pws);
-				/* Value is pre-hashed, no work to do for this value */
-				continue;
-			} else if (! slapd_nss_is_initialized() ) {
-				/* We need to hash a value but NSS is not initialized; bail */
-				return -1;
+		if (mod->mod_bvalues != NULL) {
+			for (j = 0; mod->mod_bvalues[j] != NULL; j++) {
+				char *val = mod->mod_bvalues[j]->bv_val;
+				char *hashedval = NULL;
+				struct pw_scheme *pws = pw_val2scheme (val, NULL, 0);
+				if (pws) {
+					free_pw_scheme(pws);
+					/* Value is pre-hashed, no work to do for this value */
+					continue;
+				} else if (! slapd_nss_is_initialized() ) {
+					/* We need to hash a value but NSS is not initialized; bail */
+					return -1;
+				}
+				hashedval=(slapdFrontendConfig->rootpwstoragescheme->pws_enc)(val);
+				slapi_ch_free_string (&val);
+				mod->mod_bvalues[j]->bv_val = hashedval;
+				mod->mod_bvalues[j]->bv_len = strlen (hashedval);
 			}
-			hashedval=(slapdFrontendConfig->rootpwstoragescheme->pws_enc)(val);
-			slapi_ch_free_string (&val);
-			mod->mod_bvalues[j]->bv_val = hashedval;
-			mod->mod_bvalues[j]->bv_len = strlen (hashedval);
 		}
 	}
 	return 0;
