@@ -234,8 +234,8 @@ void free_pw_scheme(struct pw_scheme *pwsp)
 {
 	if ( pwsp != NULL )
 	{
-		slapi_ch_free( (void**)&pwsp->pws_name );
-		slapi_ch_free( (void**)&pwsp );
+		slapi_ch_free_string(&pwsp->pws_name);
+		slapi_ch_free((void**)&pwsp);
 	}
 }
 
@@ -611,6 +611,13 @@ update_pw_info ( Slapi_PBlock *pb , char *old_pw)
 	cur_time = current_time();
 	slapi_mods_init(&smods, 0);
 	
+	if (slapi_entry_attr_hasvalue(e, SLAPI_ATTR_OBJECTCLASS, "shadowAccount")) {
+		time_t ctime = cur_time / _SEC_PER_DAY;
+		timestr = slapi_ch_smprintf("%ld", ctime);
+		slapi_mods_add_string(&smods, LDAP_MOD_REPLACE, "shadowLastChange", timestr);
+		slapi_ch_free_string(&timestr);
+	}
+
 	/* update passwordHistory */
 	if ( old_pw != NULL && pwpolicy->pw_history == 1 ) {
 		(void)update_pw_history(pb, sdn, old_pw);
@@ -699,7 +706,7 @@ update_pw_info ( Slapi_PBlock *pb , char *old_pw)
 
 	timestr = format_genTime ( pw_exp_date );
 	slapi_mods_add_string(&smods, LDAP_MOD_REPLACE, "passwordExpirationTime", timestr);
-	slapi_ch_free((void **)&timestr);
+	slapi_ch_free_string(&timestr);
 	
 	slapi_mods_add_string(&smods, LDAP_MOD_REPLACE, "passwordExpWarned", "0");
 
@@ -722,8 +729,7 @@ check_pw_minage ( Slapi_PBlock *pb, const Slapi_DN *sdn, struct berval **vals)
 	pwpolicy = new_passwdPolicy(pb, dn);
 	slapi_pblock_get ( pb, SLAPI_PWPOLICY, &pwresponse_req );
 
-	if ( !pb->pb_op->o_isroot && 
-		pwpolicy->pw_minage != 0 ) {
+	if (!pb->pb_op->o_isroot && pwpolicy->pw_minage) {
 
 		Slapi_Entry     *e;
 		char *passwordAllowChangeTime;
@@ -753,9 +759,7 @@ check_pw_minage ( Slapi_PBlock *pb, const Slapi_DN *sdn, struct berval **vals)
 					slapi_pwpolicy_make_response_control ( pb, -1, -1,
 							LDAP_PWPOLICY_PWDTOOYOUNG );
 				}
-				pw_send_ldap_result ( pb,
-                        LDAP_CONSTRAINT_VIOLATION, NULL,
-                        "within password minimum age", 0, NULL );
+				pw_send_ldap_result(pb, LDAP_CONSTRAINT_VIOLATION, NULL, "within password minimum age", 0, NULL);
 				slapi_entry_free( e );
 				slapi_ch_free((void **) &cur_time_str );
 				return ( 1 );
@@ -787,9 +791,9 @@ check_pw_syntax_ext ( Slapi_PBlock *pb, const Slapi_DN *sdn, Slapi_Value **vals,
 	char			*dn= (char*)slapi_sdn_get_ndn(sdn); /* jcm - Had to cast away const */
 	char			*pwd = NULL;
 	char			*p = NULL;
-	char			errormsg[ BUFSIZ ];
 	passwdPolicy	*pwpolicy = NULL;
 	Slapi_Operation *operation = NULL;
+	char errormsg[SLAPI_DSE_RETURNTEXT_SIZE] = {0};
 
 	/*
 	 * check_pw_syntax_ext could be called with mod_op == LDAP_MOD_DELETE.
@@ -834,11 +838,9 @@ check_pw_syntax_ext ( Slapi_PBlock *pb, const Slapi_DN *sdn, Slapi_Value **vals,
 			if (!is_replication && !config_get_allow_hashed_pw() && 
 				((internal_op && pb->pb_conn && !slapi_dn_isroot(pb->pb_conn->c_dn)) || 
 				(!internal_op && !pw_is_pwp_admin(pb, pwpolicy)))) {
-				PR_snprintf( errormsg, BUFSIZ,
-					"invalid password syntax - passwords with storage scheme are not allowed");
+				PR_snprintf( errormsg, sizeof(errormsg) - 1, "invalid password syntax - passwords with storage scheme are not allowed");
 				if ( pwresponse_req == 1 ) {
-					slapi_pwpolicy_make_response_control ( pb, -1, -1,
-							LDAP_PWPOLICY_INVALIDPWDSYNTAX );
+					slapi_pwpolicy_make_response_control ( pb, -1, -1, LDAP_PWPOLICY_INVALIDPWDSYNTAX );
 				}
 				pw_send_ldap_result ( pb, LDAP_CONSTRAINT_VIOLATION, NULL, errormsg, 0, NULL );
 				return( 1 );
@@ -849,7 +851,7 @@ check_pw_syntax_ext ( Slapi_PBlock *pb, const Slapi_DN *sdn, Slapi_Value **vals,
 		}
 	}
 
-	if ( pwpolicy->pw_syntax == 1 ) {
+	if ( pwpolicy->pw_syntax == LDAP_ON ) {
 		for ( i = 0; vals[ i ] != NULL; ++i ) {
 			int syntax_violation = 0;
 			int num_digits = 0;
@@ -866,8 +868,7 @@ check_pw_syntax_ext ( Slapi_PBlock *pb, const Slapi_DN *sdn, Slapi_Value **vals,
 			if ( pwpolicy->pw_minlength >
 				ldap_utf8characters((char *)slapi_value_get_string( vals[i] )) )
 			{
-				PR_snprintf( errormsg, BUFSIZ,
-				    "invalid password syntax - password must be at least %d characters long",
+				PR_snprintf( errormsg, sizeof(errormsg) - 1, "invalid password syntax - password must be at least %d characters long",
 				    pwpolicy->pw_minlength );
 				if ( pwresponse_req == 1 ) {
 					slapi_pwpolicy_make_response_control ( pb, -1, -1,
@@ -940,42 +941,42 @@ check_pw_syntax_ext ( Slapi_PBlock *pb, const Slapi_DN *sdn, Slapi_Value **vals,
 			/* check for character based syntax limits */
 			if ( pwpolicy->pw_mindigits > num_digits ) {
 				syntax_violation = 1;
-				PR_snprintf ( errormsg, BUFSIZ,
-                                    "invalid password syntax - password must contain at least %d digit characters",
-                                    pwpolicy->pw_mindigits );
+				PR_snprintf ( errormsg, sizeof(errormsg) - 1,
+				    "invalid password syntax - password must contain at least %d digit characters",
+				    pwpolicy->pw_mindigits );
 			} else if ( pwpolicy->pw_minalphas > num_alphas ) {
 				syntax_violation = 1;
-				PR_snprintf ( errormsg, BUFSIZ,
+				PR_snprintf ( errormsg, sizeof(errormsg) - 1,
 				    "invalid password syntax - password must contain at least %d alphabetic characters",
 				    pwpolicy->pw_minalphas );
 			} else if ( pwpolicy->pw_minuppers > num_uppers ) {
 				syntax_violation = 1;
-				PR_snprintf ( errormsg, BUFSIZ,
+				PR_snprintf ( errormsg, sizeof(errormsg) - 1,
 				    "invalid password syntax - password must contain at least %d uppercase characters",
 				    pwpolicy->pw_minuppers );
 			} else if ( pwpolicy->pw_minlowers > num_lowers ) {
 				syntax_violation = 1;
-				PR_snprintf ( errormsg, BUFSIZ,
+				PR_snprintf ( errormsg, sizeof(errormsg) - 1,
 				    "invalid password syntax - password must contain at least %d lowercase characters",
-					pwpolicy->pw_minlowers );
+				    pwpolicy->pw_minlowers );
 			} else if ( pwpolicy->pw_minspecials > num_specials ) {
 				syntax_violation = 1;
-				PR_snprintf ( errormsg, BUFSIZ,
+				PR_snprintf ( errormsg, sizeof(errormsg) - 1,
 				    "invalid password syntax - password must contain at least %d special characters",
 				    pwpolicy->pw_minspecials );
 			} else if ( pwpolicy->pw_min8bit > num_8bit ) {
 				syntax_violation = 1;
-				PR_snprintf ( errormsg, BUFSIZ,
+				PR_snprintf ( errormsg, sizeof(errormsg) - 1,
 				    "invalid password syntax - password must contain at least %d 8-bit characters",
 				    pwpolicy->pw_min8bit );
 			} else if ( (pwpolicy->pw_maxrepeats != 0) && (pwpolicy->pw_maxrepeats < (max_repeated + 1)) ) {
 				syntax_violation = 1;
-				PR_snprintf ( errormsg, BUFSIZ,
+				PR_snprintf ( errormsg, sizeof(errormsg) - 1,
 				    "invalid password syntax - a character cannot be repeated more than %d times",
 				    (pwpolicy->pw_maxrepeats + 1) );
 			} else if ( pwpolicy->pw_mincategories > num_categories ) {
 				syntax_violation = 1;
-				PR_snprintf ( errormsg, BUFSIZ,
+				PR_snprintf ( errormsg, sizeof(errormsg) - 1,
 				    "invalid password syntax - password must contain at least %d character "
 				    "categories (valid categories are digit, uppercase, lowercase, special, and 8-bit characters)",
 				    pwpolicy->pw_mincategories );
@@ -1053,7 +1054,7 @@ retry:
 	}
 
 	/* check for trivial words if syntax checking is enabled */
-	if ( pwpolicy->pw_syntax == 1 ) {
+	if ( pwpolicy->pw_syntax == LDAP_ON ) {
 		/* e is null if this is an add operation*/
 		if ( check_trivial_words ( pb, e, vals, "uid", pwpolicy->pw_mintokenlength, smods ) == 1 ||
 			check_trivial_words ( pb, e, vals, "cn", pwpolicy->pw_mintokenlength, smods ) == 1 ||
@@ -1335,69 +1336,6 @@ slapi_add_pwd_control ( Slapi_PBlock *pb, char *arg, long time) {
 	return( 0 );
 }
 
-void
-pw_mod_allowchange_aci(int pw_prohibit_change)
-{
-	const Slapi_DN *base;
-	char		*values_mod[2];
-	LDAPMod		mod;
-	LDAPMod		*mods[2];
-	Slapi_Backend *be;
-	char *cookie = NULL;
-
-	mods[0] = &mod;
-	mods[1] = NULL;
-	mod.mod_type = "aci";
-	mod.mod_values = values_mod;
-
-	if (pw_prohibit_change) {
-		mod.mod_op = LDAP_MOD_ADD;
-	}
-	else
-	{
-		/* Allow change password by default  */
-		/* remove the aci if it is there.  it is ok to fail */
-		mod.mod_op = LDAP_MOD_DELETE;
-	}
-
-	be = slapi_get_first_backend (&cookie);
-	/* Foreach backend... */
-    while (be)
-    {
-		/* Don't add aci on a chaining backend holding remote entries */
-        if((!be->be_private) && (!slapi_be_is_flag_set(be,SLAPI_BE_FLAG_REMOTE_DATA)))
-        {
-			/* There's only One suffix per DB now. No need to loop */
-			base = slapi_be_getsuffix(be, 0);
-			if (base != NULL)
-			{
-				Slapi_PBlock pb;
-				int rc;
-				
-				pblock_init (&pb);
-				values_mod[0] = DENY_PW_CHANGE_ACI;
-				values_mod[1] = NULL;
-				slapi_modify_internal_set_pb_ext(&pb, base, mods, NULL, NULL,
-				                                 pw_get_componentID(), 0);
-				slapi_modify_internal_pb(&pb);
-				slapi_pblock_get(&pb, SLAPI_PLUGIN_INTOP_RESULT, &rc);
-				if (rc == LDAP_SUCCESS){
-					/* 
-					** Since we modified the acl 
-					** successfully, let's update the 
-					** in-memory acl list
-					*/
-					slapi_pblock_set(&pb, SLAPI_TARGET_SDN, (void *)base);
-					plugin_call_acl_mods_update (&pb, LDAP_REQ_MODIFY );
-				}
-				pblock_done(&pb);
-			}
-        }
-		be = slapi_get_next_backend (cookie);
-    }
-	slapi_ch_free((void **) &cookie);
-}
-
 void
 add_password_attrs( Slapi_PBlock *pb, Operation *op, Slapi_Entry *e )
 {
@@ -1408,17 +1346,23 @@ add_password_attrs( Slapi_PBlock *pb, Operation *op, Slapi_Entry *e )
 	const char *dn = slapi_entry_get_ndn(e);
 	int has_allowchangetime = 0, has_expirationtime = 0;
 	time_t existing_exptime = 0;
+	time_t exptime = 0;
+	int isShadowAccount = 0;
+	int has_shadowLastChange = 0;
 
-	LDAPDebug( LDAP_DEBUG_TRACE, "add_password_attrs\n", 0, 0, 0 );
+	LDAPDebug0Args(LDAP_DEBUG_TRACE, "add_password_attrs\n");
 
 	bvals[0] = &bv;
 	bvals[1] = NULL;
-		
+
+	if (slapi_entry_attr_hasvalue(e, SLAPI_ATTR_OBJECTCLASS, "shadowAccount")) {
+		isShadowAccount = 1;
+	}
+
 	/* If passwordexpirationtime is specified by the user, don't 
 	   try to assign the initial value */
-	for ( a = &e->e_attrs; *a != NULL; a = next ) {
-		if ( strcasecmp( (*a)->a_type, 
-			"passwordexpirationtime" ) == 0) {
+	for (a = &e->e_attrs; a && *a; a = next) {
+		if (!strcasecmp((*a)->a_type, "passwordexpirationtime")) {
 			Slapi_Value *sval;
 			if (slapi_attr_first_value(*a, &sval) == 0) {
 				const struct berval *bv = slapi_value_get_berval(sval);
@@ -1426,32 +1370,44 @@ add_password_attrs( Slapi_PBlock *pb, Operation *op, Slapi_Entry *e )
 			}
 			has_expirationtime = 1;
 			
-		} else if ( strcasecmp( (*a)->a_type,
-			"passwordallowchangetime" ) == 0) {
+		} else if (!strcasecmp((*a)->a_type, "passwordallowchangetime")) {
 			has_allowchangetime = 1;
+		} else if (isShadowAccount && !strcasecmp((*a)->a_type, "shadowlastchange")) {
+			has_shadowLastChange = 1;
 		}
 		next = &(*a)->a_next;
 	}
 
-	if ( has_allowchangetime && has_expirationtime ) {
+	if (has_allowchangetime && has_expirationtime && has_shadowLastChange) {
 		return;
 	}
 
 	pwpolicy = new_passwdPolicy(pb, dn);
 
-	if ( !has_expirationtime && 
-		( pwpolicy->pw_exp || pwpolicy->pw_must_change ) ) {
-		if ( pwpolicy->pw_must_change) {
+	if (!has_expirationtime && (pwpolicy->pw_exp || pwpolicy->pw_must_change)) {
+		if (pwpolicy->pw_must_change) {
 			/* must change password when first time logon */
 			bv.bv_val = format_genTime ( NO_TIME );
 		} else if ( pwpolicy->pw_exp ) {
-			bv.bv_val = format_genTime ( time_plus_sec ( current_time (),
-       	                 pwpolicy->pw_maxage ) );
+			exptime = time_plus_sec(current_time(), pwpolicy->pw_maxage);
+			bv.bv_val = format_genTime(exptime);
 		}
 		bv.bv_len = strlen( bv.bv_val );
 		slapi_entry_attr_merge( e, "passwordexpirationtime", bvals );
 		slapi_ch_free_string( &bv.bv_val );
 	}
+	if (isShadowAccount && !has_shadowLastChange) {
+		if (pwpolicy->pw_must_change) {
+			/* must change password when first time logon */
+			bv.bv_val = slapi_ch_smprintf("0");
+		} else {
+			exptime = current_time() / _SEC_PER_DAY;
+			bv.bv_val = slapi_ch_smprintf("%ld", exptime);
+		}
+		bv.bv_len = strlen(bv.bv_val);
+		slapi_entry_attr_merge(e, "shadowLastChange", bvals);
+		slapi_ch_free_string(&bv.bv_val);
+	}
 
 	/* 
 	 * If the password minimum age is not 0, calculate when the password 
@@ -1463,12 +1419,11 @@ add_password_attrs( Slapi_PBlock *pb, Operation *op, Slapi_Entry *e )
 	 */
 	if ( !has_allowchangetime && pwpolicy->pw_minage != 0 && 
 		(has_expirationtime && existing_exptime > current_time()) ) {
-		bv.bv_val = format_genTime ( time_plus_sec ( current_time (),
-                        pwpolicy->pw_minage ) );
+		bv.bv_val = format_genTime ( time_plus_sec ( current_time (), pwpolicy->pw_minage ) );
 		bv.bv_len = strlen( bv.bv_val );
-	
+
 		slapi_entry_attr_merge( e, "passwordallowchangetime", bvals );
-		slapi_ch_free((void **) &bv.bv_val );
+		slapi_ch_free_string( &bv.bv_val );
 	}
 }
 
@@ -1565,24 +1520,6 @@ check_trivial_words (Slapi_PBlock *pb, Slapi_Entry *e, Slapi_Value **vals, char
 	return ( 0 );
 }
 
-
-void
-pw_add_allowchange_aci(Slapi_Entry *e, int pw_prohibit_change) {
-	char		*aci_pw = NULL;
-	const char *aciattr = "aci";
-
-	aci_pw = slapi_ch_strdup(DENY_PW_CHANGE_ACI);
-
-	if (pw_prohibit_change) {
-		/* Add ACI */
-		slapi_entry_add_string(e, aciattr, aci_pw);
-	} else {
-		/* Remove ACI */
-		slapi_entry_delete_string(e, aciattr, aci_pw);
-	}
-	slapi_ch_free((void **) &aci_pw);
-}
-
 int
 pw_is_pwp_admin(Slapi_PBlock *pb, passwdPolicy *pwp){
 	Slapi_DN *bind_sdn = NULL;
@@ -1759,9 +1696,17 @@ new_passwdPolicy(Slapi_PBlock *pb, const char *dn)
 			attribute in the target entry itself. */
 		} else {
 			if ( (e = get_entry( pb, dn )) != NULL ) {
-				rc = slapi_vattr_values_get(e, "pwdpolicysubentry", &values,
-					&type_name_disposition, &actual_type_name, 
-					SLAPI_VIRTUALATTRS_REQUEST_POINTERS, &attr_free_flags);
+				Slapi_Attr* attr = NULL;
+				rc = slapi_entry_attr_find(e, "pwdpolicysubentry", &attr);
+				if (attr && (0 == rc)) {
+					/* If the entry has pwdpolicysubentry, use the PwPolicy. */
+					values = valueset_dup(&attr->a_present_values);
+				} else {
+					/* Otherwise, retrieve the policy from CoS Cache */
+					rc = slapi_vattr_values_get(e, "pwdpolicysubentry", &values,
+						&type_name_disposition, &actual_type_name,
+						SLAPI_VIRTUALATTRS_REQUEST_POINTERS, &attr_free_flags);
+				}
 				if (rc) {
 					values = NULL;
 				}
@@ -1803,7 +1748,7 @@ new_passwdPolicy(Slapi_PBlock *pb, const char *dn)
 				slapi_attr_get_type(attr, &attr_name);
 				if (!strcasecmp(attr_name, "passwordminage")) {
 					if ((sval = attr_get_present_values(attr))) {
-						pwdpolicy->pw_minage = slapi_value_get_timelong(*sval);
+						pwdpolicy->pw_minage = slapi_value_get_timelonglong(*sval);
 						if (-1 == pwdpolicy->pw_minage) {
 							LDAPDebug2Args(LDAP_DEBUG_ANY, 
 								"Password Policy Entry%s: Invalid passwordMinAge: %s\n",
@@ -1815,7 +1760,7 @@ new_passwdPolicy(Slapi_PBlock *pb, const char *dn)
 				else
 				if (!strcasecmp(attr_name, "passwordmaxage")) {
 					if ((sval = attr_get_present_values(attr))) {
-						pwdpolicy->pw_maxage = slapi_value_get_timelong(*sval);
+						pwdpolicy->pw_maxage = slapi_value_get_timelonglong(*sval);
 						if (-1 == pwdpolicy->pw_maxage) {
 							LDAPDebug2Args(LDAP_DEBUG_ANY, 
 								"Password Policy Entry%s: Invalid passwordMaxAge: %s\n",
@@ -1827,7 +1772,7 @@ new_passwdPolicy(Slapi_PBlock *pb, const char *dn)
 				else
 				if (!strcasecmp(attr_name, "passwordwarning")) {
 					if ((sval = attr_get_present_values(attr))) {
-						pwdpolicy->pw_warning = slapi_value_get_timelong(*sval);
+						pwdpolicy->pw_warning = slapi_value_get_timelonglong(*sval);
 						if (-1 == pwdpolicy->pw_warning) {
 							LDAPDebug2Args(LDAP_DEBUG_ANY, 
 								"Password Policy Entry%s: Invalid passwordWarning: %s\n",
@@ -2007,7 +1952,28 @@ new_passwdPolicy(Slapi_PBlock *pb, const char *dn)
 			if (pw_entry) {
 				slapi_entry_free(pw_entry);
 			}
-			if(pb){
+			if (LDAP_ON != pwdpolicy->pw_syntax) {
+				passwdPolicy *g_pwdpolicy = &(slapdFrontendConfig->pw_policy);
+				/* 
+				 * When the fine-grained password policy does not set the
+				 * password syntax, get the syntax from the global
+				 * policy if nsslapd-pwpolicy-inherit-global is on.
+				 */
+				if ((LDAP_ON == g_pwdpolicy->pw_syntax) && config_get_pwpolicy_inherit_global()) {
+					pwdpolicy->pw_minlength = g_pwdpolicy->pw_minlength;
+					pwdpolicy->pw_mindigits = g_pwdpolicy->pw_mindigits;
+					pwdpolicy->pw_minalphas = g_pwdpolicy->pw_minalphas;
+					pwdpolicy->pw_minuppers = g_pwdpolicy->pw_minuppers;
+					pwdpolicy->pw_minlowers = g_pwdpolicy->pw_minlowers;
+					pwdpolicy->pw_minspecials = g_pwdpolicy->pw_minspecials;
+					pwdpolicy->pw_min8bit = g_pwdpolicy->pw_min8bit;
+					pwdpolicy->pw_maxrepeats = g_pwdpolicy->pw_maxrepeats;
+					pwdpolicy->pw_mincategories = g_pwdpolicy->pw_mincategories;
+					pwdpolicy->pw_mintokenlength = g_pwdpolicy->pw_mintokenlength;
+					pwdpolicy->pw_syntax = LDAP_ON; /* Need to enable it to apply the default values */
+				}
+			}
+			if (pb) {
 				pb->pwdpolicy = pwdpolicy;
 			}
 			return pwdpolicy;
@@ -2158,33 +2124,38 @@ pw_boolean_str2value (const char *str)
 }
 
 int
-check_pw_duration_value( const char *attr_name, char *value, 
-                       long minval, long maxval, char *errorbuf )
+check_pw_duration_value(const char *attr_name, char *value,
+                        long minval, long maxval, char *errorbuf, size_t ebuflen)
 {
 	int retVal = LDAP_SUCCESS;
-	long age;
+	long long age;
 
-	age = parse_duration(value);
+	age = slapi_parse_duration_longlong(value);
 	if (-1 == age) {
-		PR_snprintf ( errorbuf, BUFSIZ, 
-		              "password minimum age \"%s\" is invalid. ", value );
+		slapi_create_errormsg(errorbuf, ebuflen, "password minimum age \"%s\" is invalid. ", value);
 		retVal = LDAP_CONSTRAINT_VIOLATION;
 	} else if (0 == strcasecmp(CONFIG_PW_LOCKDURATION_ATTRIBUTE, attr_name)) {
 		if ( (age <= 0) ||
+#if defined(CPU_x86_64)
+			 (age > (MAX_ALLOWED_TIME_IN_SECS_64 - current_time())) ||
+#else
 			 (age > (MAX_ALLOWED_TIME_IN_SECS - current_time())) ||
+#endif
 			 ((-1 != minval) && (age < minval)) ||
 			 ((-1 != maxval) && (age > maxval))) {
-			PR_snprintf ( errorbuf, BUFSIZ, "%s: \"%s\" seconds is invalid. ",
-			              attr_name, value );
+			slapi_create_errormsg(errorbuf, ebuflen, "%s: \"%s\" seconds is invalid. ", attr_name, value);
 			retVal = LDAP_CONSTRAINT_VIOLATION;
 		}
 	} else {
 		if ( (age < 0) ||
+#if defined(CPU_x86_64)
+			 (age > (MAX_ALLOWED_TIME_IN_SECS_64 - current_time())) ||
+#else
 			 (age > (MAX_ALLOWED_TIME_IN_SECS - current_time())) ||
+#endif
 			 ((-1 != minval) && (age < minval)) ||
 			 ((-1 != maxval) && (age > maxval))) {
-			PR_snprintf ( errorbuf, BUFSIZ, "%s: \"%s\" seconds is invalid. ",
-			              attr_name, value );
+			slapi_create_errormsg(errorbuf, ebuflen, "%s: \"%s\" seconds is invalid. ", attr_name, value);
 			retVal = LDAP_CONSTRAINT_VIOLATION;
 		}
 	}
@@ -2193,7 +2164,8 @@ check_pw_duration_value( const char *attr_name, char *value,
 }
 
 int
-check_pw_resetfailurecount_value( const char *attr_name, char *value, long minval, long maxval, char *errorbuf )
+check_pw_resetfailurecount_value(const char *attr_name, char *value,
+                                 long minval, long maxval, char *errorbuf, size_t ebuflen)
 {
 	int retVal = LDAP_SUCCESS;
 	long duration = 0; /* in minutes */
@@ -2201,9 +2173,7 @@ check_pw_resetfailurecount_value( const char *attr_name, char *value, long minva
 	/* in seconds */  
 	duration = strtol (value, NULL, 0);
 	if ( duration < 0 || duration > (MAX_ALLOWED_TIME_IN_SECS - current_time()) ) {
-		PR_snprintf ( errorbuf, BUFSIZ, 
-			"password reset count duration \"%s\" seconds is invalid. ",
-			value );
+		slapi_create_errormsg(errorbuf, ebuflen, "password reset count duration \"%s\" seconds is invalid.", value);
 		retVal = LDAP_CONSTRAINT_VIOLATION;
 	}
 
@@ -2211,7 +2181,8 @@ check_pw_resetfailurecount_value( const char *attr_name, char *value, long minva
 }
 
 int
-check_pw_storagescheme_value( const char *attr_name, char *value, long minval, long maxval, char *errorbuf )
+check_pw_storagescheme_value(const char *attr_name, char *value,
+                             long minval, long maxval, char *errorbuf, size_t ebuflen)
 {
 	int retVal = LDAP_SUCCESS;
 	struct pw_scheme *new_scheme = NULL;
@@ -2221,16 +2192,13 @@ check_pw_storagescheme_value( const char *attr_name, char *value, long minval, l
 	new_scheme = pw_name2scheme(value);
 	if ( new_scheme == NULL) {
 		if ( scheme_list != NULL ) {
-			PR_snprintf ( errorbuf, BUFSIZ,
-					"%s: invalid scheme - %s. Valid schemes are: %s",
+			slapi_create_errormsg(errorbuf, ebuflen, "%s: invalid scheme - %s. Valid schemes are: %s",
 					CONFIG_PW_STORAGESCHEME_ATTRIBUTE, value, scheme_list );
 		} else {
-			PR_snprintf ( errorbuf, BUFSIZ,
-					"%s: invalid scheme - %s (no pwdstorage scheme"
-					" plugin loaded)",
+			slapi_create_errormsg(errorbuf, ebuflen, "%s: invalid scheme - %s (no pwdstorage scheme plugin loaded)",
 					CONFIG_PW_STORAGESCHEME_ATTRIBUTE, value);
 		}
-	retVal = LDAP_CONSTRAINT_VIOLATION;
+		retVal = LDAP_CONSTRAINT_VIOLATION;
 	}
 	else if ( new_scheme->pws_enc == NULL )
 	{
@@ -2240,9 +2208,8 @@ check_pw_storagescheme_value( const char *attr_name, char *value, long minval, l
 		and won't encrypt passwords if they are in clear. We don't take it 
 		*/ 
 
-		if ( scheme_list != NULL ) {
-			PR_snprintf ( errorbuf, BUFSIZ, 
-				"%s: invalid encoding scheme - %s\nValid values are: %s\n",
+		if (scheme_list) {
+			slapi_create_errormsg(errorbuf, ebuflen, "%s: invalid encoding scheme - %s\nValid values are: %s\n",
 				CONFIG_PW_STORAGESCHEME_ATTRIBUTE, value, scheme_list );
 		}
 
@@ -2829,3 +2796,166 @@ pw_get_ext_size(Slapi_Entry *entry, size_t *size)
     }
     return LDAP_SUCCESS;
 }
+
+int
+add_shadow_ext_password_attrs(Slapi_PBlock *pb, Slapi_Entry **e)
+{
+    const char *dn = NULL;
+    passwdPolicy *pwpolicy = NULL;
+    long long shadowval = 0;
+    long long exptime = 0;
+    Slapi_Mods *smods = NULL;
+    LDAPMod **mods;
+    long long sval;
+    int mod_num = 0;
+    char *shmin = NULL;
+    char *shmax = NULL;
+    char *shwarn = NULL;
+    char *shexp = NULL;
+    int rc = 0;
+
+    if (!e || !*e) {
+        return rc;
+    }
+    dn = slapi_entry_get_ndn(*e);
+    if (!dn) {
+        return rc;
+    }
+    if (!slapi_entry_attr_hasvalue(*e, SLAPI_ATTR_OBJECTCLASS, "shadowAccount")) {
+        /* Not a shadowAccount; nothing to do. */
+        return rc;
+    }
+    if (operation_is_flag_set(pb->pb_op, OP_FLAG_INTERNAL)) {
+        /* external only */
+        return rc;
+    }
+    pwpolicy = new_passwdPolicy(pb, dn);
+    if (!pwpolicy) {
+        return rc;
+    }
+
+    LDAPDebug0Args(LDAP_DEBUG_TRACE, "--> add_shadow_password_attrs\n");
+
+    /* shadowMin - the minimum number of days required between password changes. */
+    if (pwpolicy->pw_minage > 0) {
+        shadowval = pwpolicy->pw_minage / _SEC_PER_DAY;
+        if (shadowval > _MAX_SHADOW) {
+            shadowval = _MAX_SHADOW;
+        }
+    } else {
+        shadowval = 0;
+    }
+    shmin = slapi_entry_attr_get_charptr(*e, "shadowMin");
+    if (shmin) {
+        sval = strtoll(shmin, NULL, 0);
+        if (sval != shadowval) {
+            slapi_ch_free_string(&shmin);
+            shmin = slapi_ch_smprintf("%lld", shadowval);
+            mod_num++;
+        }
+    } else {
+        mod_num++;
+        shmin = slapi_ch_smprintf("%lld", shadowval);
+    }
+
+    /* shadowMax - the maximum number of days for which the user password remains valid. */
+    if (pwpolicy->pw_maxage > 0) {
+        shadowval = pwpolicy->pw_maxage / _SEC_PER_DAY;
+        exptime = time_plus_sec(current_time(), pwpolicy->pw_maxage);
+        if (shadowval > _MAX_SHADOW) {
+            shadowval = _MAX_SHADOW;
+        }
+    } else {
+        shadowval = _MAX_SHADOW;
+    }
+    shmax = slapi_entry_attr_get_charptr(*e, "shadowMax");
+    if (shmax) {
+        sval = strtoll(shmax, NULL, 0);
+        if (sval != shadowval) {
+            slapi_ch_free_string(&shmax);
+            shmax = slapi_ch_smprintf("%lld", shadowval);
+            mod_num++;
+        }
+    } else {
+        mod_num++;
+        shmax = slapi_ch_smprintf("%lld", shadowval);
+    }
+
+    /* shadowWarning - the number of days of advance warning given to the user before the user password expires. */
+    if (pwpolicy->pw_warning > 0) {
+        shadowval = pwpolicy->pw_warning / _SEC_PER_DAY;
+        if (shadowval > _MAX_SHADOW) {
+            shadowval = _MAX_SHADOW;
+        }
+    } else {
+        shadowval = 0;
+    }
+    shwarn = slapi_entry_attr_get_charptr(*e, "shadowWarning");
+    if (shwarn) {
+        sval = strtoll(shwarn, NULL, 0);
+        if (sval != shadowval) {
+            slapi_ch_free_string(&shwarn);
+            shwarn = slapi_ch_smprintf("%lld", shadowval);
+            mod_num++;
+        }
+    } else {
+        mod_num++;
+        shwarn = slapi_ch_smprintf("%lld", shadowval);
+    }
+
+    /* shadowExpire - the date on which the user login will be disabled. */
+    if (exptime) {
+        shexp = slapi_entry_attr_get_charptr(*e, "shadowExpire");
+        exptime /= _SEC_PER_DAY;
+        if (shexp) {
+            sval = strtoll(shexp, NULL, 0);
+            if (sval != exptime) {
+                slapi_ch_free_string(&shexp);
+                shexp = slapi_ch_smprintf("%lld", exptime);
+                mod_num++;
+            }
+        } else {
+            mod_num++;
+            shexp = slapi_ch_smprintf("%lld", exptime);
+        }
+    }
+    smods = slapi_mods_new();
+    slapi_mods_init(smods, mod_num);
+    if (shmin) {
+        slapi_mods_add(smods, LDAP_MOD_REPLACE, "shadowMin", strlen(shmin), shmin);
+        slapi_ch_free_string(&shmin);
+    }
+    if (shmax) {
+        slapi_mods_add(smods, LDAP_MOD_REPLACE, "shadowMax", strlen(shmax), shmax);
+        slapi_ch_free_string(&shmax);
+    }
+    if (shwarn) {
+        slapi_mods_add(smods, LDAP_MOD_REPLACE, "shadowWarning", strlen(shwarn), shwarn);
+        slapi_ch_free_string(&shwarn);
+    }
+    if (shexp) {
+        slapi_mods_add(smods, LDAP_MOD_REPLACE, "shadowExpire", strlen(shexp), shexp);
+        slapi_ch_free_string(&shexp);
+    }
+    /* Apply the  mods to create the resulting entry. */
+    mods = slapi_mods_get_ldapmods_byref(smods);
+    if (mods) {
+        Slapi_Entry *sentry = slapi_entry_dup(*e);
+        rc = slapi_entry_apply_mods(sentry, mods);
+        pb->pb_pw_entry = sentry;
+        *e = sentry;
+    }
+    slapi_mods_free(&smods);
+
+#if 0 /* These 2 attributes are no need (or not able) to auto-fill. */
+    /* 
+     * shadowInactive - the number of days of inactivity allowed for the user.
+     * Password Policy does not have the corresponding parameter.
+     * 
+     * shadowFlag - not currently in use.
+     */
+#endif
+
+    LDAPDebug0Args(LDAP_DEBUG_TRACE, "<-- add_shadow_password_attrs\n");
+    return rc;
+}
