@@ -682,14 +682,14 @@ PRBool slapi_mapping_tree_node_is_set (const mapping_tree_node *node,
 	PRUint32 flag); 
 Slapi_DN* slapi_mtn_get_dn(mapping_tree_node *node);
 int slapi_mapping_tree_select_and_check(Slapi_PBlock *pb,char *newdn,
-	Slapi_Backend **be, Slapi_Entry **referral, char *errorbuf);
+	Slapi_Backend **be, Slapi_Entry **referral, char *errorbuf, size_t ebuflen);
 int slapi_mapping_tree_select_all(Slapi_PBlock *pb, Slapi_Backend **be_list,
-	Slapi_Entry **referral_list, char *errorbuf);
+	Slapi_Entry **referral_list, char *errorbuf, size_t ebuflen);
 void slapi_mapping_tree_free_all(Slapi_Backend **be_list,
 	Slapi_Entry **referral_list);
 
 /* Mapping Tree */
-int slapi_mapping_tree_select(Slapi_PBlock *pb, Slapi_Backend **be, Slapi_Entry **referral, char *error_string);
+int slapi_mapping_tree_select(Slapi_PBlock *pb, Slapi_Backend **be, Slapi_Entry **referral, char *error_string, size_t ebuflen);
 char ** slapi_mtn_get_referral(const Slapi_DN *sdn);
 int slapi_mtn_set_referral(const Slapi_DN *sdn, char ** referral);
 int slapi_mtn_set_state(const Slapi_DN *sdn, char *state);
@@ -1161,6 +1161,7 @@ char* slapd_get_tmp_dir( void );
 #include <stdio.h> /* GGOODREPL - For BUFSIZ, below, gak */
 const char* escape_string (const char* str, char buf[BUFSIZ]);
 const char* escape_string_with_punctuation(const char* str, char buf[BUFSIZ]);
+const char* escape_string_for_filename(const char* str, char buf[BUFSIZ]);
 void strcpy_unescape_value( char *d, const char *s );
 
 char *slapi_berval_get_string_copy(const struct berval *bval);
@@ -1305,6 +1306,8 @@ void add_internal_modifiersname(Slapi_PBlock *pb, Slapi_Entry *e);
 
 /* ldaputil.c */
 char *ldaputil_get_saslpath();
+int slapi_client_uses_non_nss(LDAP *ld);
+int slapi_client_uses_openssl(LDAP *ld);
 
 /* ssl.c */
 /* 
@@ -1333,8 +1336,39 @@ char *slapi_getSSLVersion_str(PRUint16 vnum, char *buf, size_t bufsize);
  *    Failure: -1
  */
 time_t slapi_parse_duration(const char *value);
+long long slapi_parse_duration_longlong(const char *value);
 int slapi_is_duration_valid(const char *value);
 
+/**
+ * Populate the pointers with the system memory information.
+ * At this time, Linux is the only "reliable" system for returning these values
+ *
+ * \param pagesize Will return the system page size in bytes.
+ * \param pages The total number of memory pages on the system. May include swap pages depending on OS.
+ * \param procpages Number of memory pages our current process is consuming. May not be accurate on all platforms as this could be the VMSize rather than the actual number of consumed pages.
+ * \param availpages Number of available pages of memory on the system. Not all operating systems set this correctly.
+ *
+ * \return 0 on success, non-zero on failure to determine memory sizings.
+ */
+int util_info_sys_pages(size_t *pagesize, size_t *pages, size_t *procpages, size_t *availpages);
+
+/**
+ * Determine if the requested cachesize will exceed the system memory limits causing an out of memory condition
+ *
+ * \param cachesize. The requested allocation. If this value is greater than the memory available, this value will be REDUCED to be valid.
+ *
+ * \return 0 if the size is "sane". 1 if the value will cause OOM and has been REDUCED
+ */
+int util_is_cachesize_sane(size_t *cachesize);
+
+/**
+ * Write an error message to the given error buffer.
+ *
+ * \param errorbuf. The buffer that the error message is written into.  If NULL, nothing happens.  It could be a static array or allocated memory.  If it is allocated memory, the next param len should be given.
+ * \param len. The length of errorbuf.  If 0 is given, sizeof(errorbuf) is used. 
+ * \param fmt. The format of the error message.
+ */
+void slapi_create_errormsg(char *errorbuf, size_t len, const char *fmt, ...);
 
 #ifdef __cplusplus
 }
